import {
  AddressType,
  CandidateType,
  OrganizationType,
  Reducers,
  USER_TYPE,
} from "../types";

export const logger = (log: any, ...optionalparams: any[]) => {
  console.log(log, ...optionalparams);
};

// 16 - Shift
// 17 - Ctrl
// 18 - Alt
// 8 - Backspace
export function checkUnwantedKey(keycode: number) {
  return [16, 17, 18].includes(keycode);
}

export function keyEvent(e: any) {
  // Ignore keypresses for unwanted keys
  if (checkUnwantedKey(e.keyCode)) {
    return null;
  }

  // Ignore keypresses with additional modifiers
  if (e.altKey || e.ctrlKey) {
    return null;
  }

  return {
    key: e.key,
    code: e.code,
    shiftKey: e.shiftKey,
    time: Date.now(),
  };
}

export const getProfile = (
  state: Reducers
): CandidateType | OrganizationType | {} => {
  if (state.auth.userType === USER_TYPE.CANDIDATE) {
    return state.auth.candidate;
  }
  return state.auth.organization;
};

export const getUserId = (state: Reducers): string => {
  const profile: any = getProfile(state);

  return profile._id as string;
};

export const getStatusColor = (status?: string) => {
  const color =
    status === "Accepted"
      ? "text-bg-success"
      : status === "Pending"
      ? "text-bg-warning"
      : status === "Rejected"
      ? "text-bg-danger"
      : "text-bg-primary";
  return color;
};

export const getAddress = (address: AddressType) => {
  if (!address) return "";
  return `${address.addressLine}, ${address.city}, ${address.country}`;
};

export const getVerificationColor = (score: number) => {
  const color =
    score >= 80
      ? "text-success"
      : score >= 60 && score < 80
      ? "text-warning"
      : "text-danger";
  return color;
};
