import { put, takeLeading, call, select } from "redux-saga/effects";
import { ACTIONS } from "../actions";
import CommonAPI from "../apis/common";
import UserAPI from "../apis/user";
import { getProfile } from "../lib/util";
import { APP_STATE, CandidateType, JobType, Reducers } from "../types";

function* updateCandidate({
  payload,
}: {
  type: typeof ACTIONS.UPDATE_CANDIDATE;
  payload: Partial<CandidateType>;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    yield call(UserAPI.updateCandidate, payload);
    const candidate: CandidateType = yield select(getProfile);
    yield put({
      type: ACTIONS.SET_AUTH,
      payload: { candidate: { ...candidate, ...payload } },
    });

    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

function* applyForJob({
  payload,
}: {
  type: typeof ACTIONS.APPLY_FOR_JOB;
  payload: string;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });

    const profile: CandidateType = yield select(getProfile);

    if (profile._id && profile.resume) {
      const data: { applicationId: string; success: boolean } = yield call(
        CommonAPI.applyForJob,
        {
          application: {
            candidate: profile._id,
            job: payload,
            status: "Pending",
            interview: {
              date: "",
              time: "",
              link: "",
              voiceVerification: "Pending",
            },
          },
          resumeUrl: profile.resume,
        }
      );

      if (data.success) {
        let jobs: JobType[] = yield select(
          (state: Reducers) => state.common.jobs
        );
        const found = jobs.findIndex((_job) => _job._id === payload);
        jobs[found] = {
          ...jobs[found],
          applications: [
            ...(jobs[found]?.applications || []),
            data.applicationId,
          ],
        };

        yield put({ type: ACTIONS.SET_JOBS, payload: [...jobs] });
      }
    }

    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

export default function* authSaga() {
  yield takeLeading(ACTIONS.UPDATE_CANDIDATE, updateCandidate);
  yield takeLeading(ACTIONS.APPLY_FOR_JOB, applyForJob);
}
