import React, { useState, useEffect, ChangeEvent, useMemo } from "react";
import { useOutletContext, useSearchParams } from "react-router-dom";

import { ApplicationPayloadType, JobPayloadType } from "../common/types";
import TabNavBar from "../components/TabNavBar";
import { useDispatch } from "react-redux";
import { updateApplication } from "../common/actions/common";
import Applicant from "../components/Application/Applicant";
import Interview from "../components/Application/Interview";

const tabs = ["Candidate", "Interview"];

const Application = () => {
  const dispatch = useDispatch();
  const [searchParams] = useSearchParams();
  const [selectedTab, setSelectedTab] = useState("Candidate");
  const job = useOutletContext<JobPayloadType>();
  const applicationId = searchParams.get("applicationId");
  const application = (job.applications as ApplicationPayloadType[])?.find(
    (_application) => _application._id === applicationId
  );
  const [status, setStatus] = useState(application?.status);

  const statusValues = useMemo(() => {
    if (status === "Pending") {
      return ["Schedule", "Reject"];
    } else if (status === "In progress") {
      return ["Accept", "Reject"];
    } else if (status === "Rejected") {
      return ["Pending", "Schedule"];
    } else {
      return [];
    }
  }, []);

  useEffect(() => {
    if (application) {
      if (application?.status === "Pending") {
        setSelectedTab("Candidate");
        setStatus(application.status);
      } else if (application?.status === "In progress") {
        setSelectedTab("Interview");
        setStatus("Schedule");
      }
    }
  }, [application]);

  if (!application) return null;

  const onUpdateStatus = () => {
    let applicationStatus = status;
    if (status === "Schedule") {
      applicationStatus = "In progress";
    } else if (status === "Reject") {
      applicationStatus = "Rejected";
    }

    console.log("STATUS ", status, applicationStatus);

    // dispatch(
    //   updateApplication({
    //     applicationId: application._id,
    //     update: { status: applicationStatus },
    //     candidateId: application.candidate._id || "",
    //   })
    // );
  };

  const onChangeStatus = (e: ChangeEvent<HTMLSelectElement>) => {
    if (e.target.value !== "select") {
      setStatus(e.target.value);
    }
  };

  const renderOptions = (option: string, index: number) => (
    <option key={index} value={option}>
      {option}
    </option>
  );

  return (
    <div className="card p-4">
      <TabNavBar tabs={tabs} selected={selectedTab} onSelect={setSelectedTab} />
      {selectedTab === "Candidate" && <Applicant application={application} />}
      {selectedTab === "Interview" && <Interview application={application} />}

      <hr />
      {statusValues.length > 0 && (
        <div className="row ">
          <label className="col-sm-2 col-form-label">Status</label>
          <div className="col-sm-6 ">
            <div className="input-group ">
              <select
                className="form-select"
                id="inputGroupSelect04"
                aria-label="Example select with button addon"
                value={status}
                onChange={onChangeStatus}
              >
                <option value="select">Select</option>
                {statusValues.map(renderOptions)}
              </select>
              <button
                className="btn btn-primary "
                type="button"
                onClick={onUpdateStatus}
              >
                Update & Next
              </button>
            </div>
          </div>
        </div>
      )}
    </div>
  );
};

export default Application;
